package org.hibernate.internal;

import org.hibernate.*;
import org.hibernate.cfg.*;
import org.hibernate.engine.spi.*;
import org.hibernate.id.*;
import org.hibernate.internal.util.*;
import org.hibernate.mapping.*;
import org.hibernate.persister.entity.*;
import org.hibernate.persister.spi.*;
import org.hibernate.service.*;
import java.util.*;
import java.util.Map;

public class MutableSessionFactoryImpl
extends SessionFactoryImpl
{
   private final static transient Map<String, String> dynamicImports =
                                                   new HashMap<String, String>();

   public MutableSessionFactoryImpl(final Configuration cfg,
                                    Mapping mapping,
                                    ServiceRegistry serviceRegistry,
                                    Settings settings,
                                    SessionFactoryObserver observer)
   throws HibernateException
   {
      super(cfg, mapping, serviceRegistry, settings, observer);
   }

   public void add(PersistentClass persistentClass, Configuration configuration)
   {
      IdentifierGenerator generator = persistentClass.getIdentifier().createIdentifierGenerator(
						configuration.getIdentifierGeneratorFactory(),
						getDialect(),
				      settings.getDefaultCatalogName(),
				      settings.getDefaultSchemaName(),
				      (RootClass) persistentClass
	   );
      identifierGenerators.put(persistentClass.getEntityName(), generator );

      EntityPersister entityPersister =
            serviceRegistry.getService(PersisterFactory.class).createEntityPersister(
                    persistentClass,
                    null, //TODO accessStrategy?,
                    null, //TODO naturalIdAccessStrategy?,
                    this,
                    configuration.getMapping());

      String entityName = persistentClass.getEntityName();
      entityPersisters.put(entityName, entityPersister );
      entityPersister.postInstantiate();
      registerEntityNameResolvers(entityPersister);

      synchronized (dynamicImports)
      {
         dynamicImports.put(entityName, entityName);
         dynamicImports.put(StringHelper.unqualify(entityName), entityName);
      }
   }

   public String getImportedClassName(String className)
   {
      String res = super.getImportedClassName(className);
      if (res == null)
      {
         synchronized (dynamicImports)
         {
            res = dynamicImports.get(className);
         }
      }

      return res;
   }
}
